package edu.upvehu.gbg.mummec2021;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;

class DBMummec2122 implements AutoCloseable{
    private static DBMummec2122 elSingleton=null;
    private static Connection con;
    private static final String DATABASE_NAME = "MUMMEC2122";
    private static PreparedStatement psPaperAuth, psInfo, psRefs, psAutFil ;
    
    private DBMummec2122() throws SQLException{
            con=DriverManager.getConnection("jdbc:mysql://localhost:3306/","german","chicle");
            elSingleton=this;
    }
    
    static DBMummec2122 getInstance() throws SQLException  {
        if (elSingleton!=null) return elSingleton;
        else return elSingleton=new DBMummec2122();
    }
   
    void instanciaEsquema() throws SQLException{
            //Se genera la base de datos destruyéndola previamente si existe. 
            con.createStatement().executeUpdate("DROP DATABASE IF EXISTS "+DATABASE_NAME+";");
            con.createStatement().executeUpdate("CREATE DATABASE "+DATABASE_NAME+";");
            con.createStatement().executeUpdate("USE "+DATABASE_NAME+";");
            
            con.setAutoCommit(false);
            con.createStatement().executeUpdate("CREATE TABLE papersreferences ( ref_paper INTEGER, cited_ref_paper INTEGER, PRIMARY KEY(ref_paper,cited_ref_paper) );");
            con.createStatement().executeUpdate("CREATE TABLE publication ( ref_publication INTEGER PRIMARY KEY, title VARCHAR(200) );");
            con.createStatement().executeUpdate("CREATE TABLE papersauthors ( ref_paper INTEGER, ref_author INTEGER, PRIMARY KEY(ref_paper,ref_author) );");
            con.createStatement().executeUpdate("CREATE TABLE authorafiliation ( ref_author INTEGER, ref_afiliation INTEGER, PRIMARY KEY(ref_author, ref_afiliation) );");
            con.createStatement().executeUpdate("CREATE TABLE afiliations ( ref_afiliation INTEGER PRIMARY KEY, university VARCHAR(65), webpage VARCHAR(30), country VARCHAR(15) );");
            con.createStatement().executeUpdate("CREATE TABLE papersinfo ( ref_paper INTEGER PRIMARY KEY, title VARCHAR(200), publiyear INTEGER, ref_publication INTEGER, url VARCHAR(200));");
            con.createStatement().executeUpdate("CREATE TABLE authors ( ref_author INTEGER PRIMARY KEY, name VARCHAR(25) );" );
            con.commit();
            
            psPaperAuth=con.prepareStatement("insert into papersauthors values (?,?);");
            psRefs=con.prepareStatement("insert into papersreferences values (?,?);");
            psAutFil=con.prepareStatement("insert into authorafiliation values (?,?);");
            psInfo=con.prepareStatement("insert into papersinfo values (?,?,?,?,?);");
    }
    
    //PARA autores, publicaciones
    void arrayOfStrings2DB(String[] valores,String tableName) throws SQLException {
        PreparedStatement ps=con.prepareStatement("insert into "+tableName+" values (?,?);");
        int i=0; for (String s:valores) {
            ps.setInt(1, ++i);
            ps.setString(2, s);
            ps.addBatch();}
        flush(ps);}
    
    //PARA afiliaciones
    void affiliations2DB(String[] valores) throws SQLException {
        PreparedStatement ps=con.prepareStatement("insert into afiliations values (?,?,?,?);");
        int i=0; for (String s:valores) {
            String[] campos=s.split("\\s*,\\s*");
            ps.setInt(1, ++i);
            ps.setString(2, campos[0]);
            ps.setString(3, campos[1]);
            ps.setString(4, campos[2]);
            ps.addBatch();
        }
        flush(ps);}

    //PARA referencias
    void flushReferences() throws SQLException {flush(psRefs);}
    void addReferences(int ref, String[] refs) throws SQLException {
        for (String s:refs) {
            psRefs.setInt(1, ref);
            psRefs.setString(2, s);
            psRefs.addBatch();}}
        
    //PARA  autor-filiación
    void flushAuthorFiliation() throws SQLException {flush(psAutFil);}
    void addAuthorFiliation(int aut, int affil) throws SQLException {
        psAutFil.setInt(1, aut);
        psAutFil.setInt(2, affil);
        psAutFil.addBatch();}

    //PARA papersInfo
    void flushInfo() throws SQLException {flush(psInfo);}
    void addInfo(int refNum, Referencia  r, int publicationNum) throws SQLException {
        psInfo.setInt(1, refNum);
        psInfo.setString(2, r.getTitulo());
        psInfo.setInt(3, Integer.parseInt(r.getAnio()));
        psInfo.setInt(4, publicationNum);
        psInfo.setString(5, r.getLink());
        psInfo.addBatch();}

    //PARA paper-autor
    void flushPaperAuth() throws SQLException {flush(psPaperAuth);} 
    void addPaperAuth(int refNum, int authornNum) throws SQLException {
        psPaperAuth.setInt(1, refNum);
        psPaperAuth.setInt(2, authornNum);
        psPaperAuth.addBatch();}

    //FLUSH
    private void flush(PreparedStatement ps) throws SQLException {ps.executeBatch();con.commit();}

    //---------------------------------
    //Implementa Interfaz Autocloseable
    @Override public void close() throws SQLException {con.close();}
  
}
