package edu.upvehu.gbg.mummec2021;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;


class Referencia {
    private static final Map<Integer, Referencia> ALMACEN=new TreeMap<>();  //mapa para almacenar todos los registros bibliográficos
    private List<String> autorFiliacionList=new ArrayList<>();              //cada registro bibliográfico tiene una lista de autor+filiación 
    private String titulo, anio, publicacion, link="NO-LINK", referencias;  //y una serie de campo textuales
    
    private Referencia(){}                                          //Los registros bibliográficos se construyen vacios
    
    public static Referencia addLine(String linea) {                //llenar el campo correspondiente del registro.
        if (linea==null) return null;
        String[] campos=linea.replaceFirst("[a-zA-Z]", " $0").split(" ",3);             //separa los tres campos de la linea ([0]numero [1]tipo [2]contenido)
        Referencia reg=ALMACEN.get(Integer.parseInt(campos[0]));                        //Se intenta coger el registro del mapa
        if (reg==null) ALMACEN.put(Integer.parseInt(campos[0]),reg=new Referencia());   //si no estaba ya, se crea uno nuevo y se deja ya metido en el mapa
        String dato=campos[2].trim();                //Esta es la informción a guardar

        switch (campos[1].charAt(0)){                //En función del tipo de información en la linea, esta se mete en el campo correspondiente
            case 't':reg.titulo=dato;break;
            case 'y':reg.anio=dato;break;
            case 'p':reg.publicacion=dato;break;
            case 'r':reg.referencias=dato;break;
            case 'a':reg.autorFiliacionList.add(dato);break;
            case 'l':reg.link=dato;break;
            default: throw new InvalidFieldException(linea);
        }
        return reg;
    }
    
    //GETTERS
    public static Map<Integer, Referencia> getAlmacen() {return ALMACEN;}
    public List<String> getAutorFiliacionList() {return autorFiliacionList;}
    public String getTitulo() {return titulo;}
    public String getAnio() {return anio;}
    public String getPublicacion() {return publicacion;}
    public String getLink() {return link;}
    public String getReferencias() {return referencias;}
}


class InvalidFieldException extends RuntimeException {
    public InvalidFieldException() {}
    public InvalidFieldException(String message) {super(message);}
}