package edu.upvehu.gbg.mummec2021;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TreeSet;

public class Solucion {

    /**
     * @param args the command line arguments
     * @throws java.io.IOException
     * @throws java.sql.SQLException
     */
    public static void main(String[] args) throws IOException, SQLException  {
        Long t0=System.currentTimeMillis();
        
        //FASE 1 - Lee toda la informacion del fichero y crea mapa
        try (BufferedReader br = new BufferedReader(
                                     new InputStreamReader(
                                         new URL("http://gtts.ehu.es/German/Docencia/assets/DB.txt").openStream()));) { 
            while (Referencia.addLine(br.readLine())!= null) /*nada*/; 
        }
        
        muestraTiempo(t0,"tras la adquisición de ("+Referencia.getAlmacen().size()+") referencias");
//======================================================================================================================

        //FASE 2.1 de generación de conjuntos con autor+filiación, autores, filiaciones y publicaciones 
        Set<String> pub=new TreeSet<>(), autFil=new TreeSet<>(), aut=new TreeSet<>(), fil=new TreeSet<>();
        for (Referencia r:Referencia.getAlmacen().values()) {      //Recorriendo todos los registros
            pub.add(r.getPublicacion());                           //Se añaden las publicaciones al conjunto
            for (String s:r.getAutorFiliacionList()) {             //y para todos los autores y filiaciones
                autFil.add(s);                                     //se guardan como tal
                String[] autAndFil=separaAutorYFiliacion(s);       //y por separado
                aut.add(autAndFil[0].trim());
                fil.add(autAndFil[1].trim());
                }
            }
        
        //FASE 2.2 autores, filiaciones y publicaciones pasan a array ordenado para establecer sus índices. 
        String[] publicaciones=pub.toArray(new String[pub.size()]), 
                       autores=aut.toArray(new String[aut.size()]),
                   filiaciones=fil.toArray(new String[fil.size()]);
        
        muestraTiempo(t0,"tras la construcción de índices");

//======================================================================================================================
        
        try (DBMummec2122 db=DBMummec2122.getInstance();) {
            db.instanciaEsquema();
            //FASE 3.1 Generación de tablas que son índices: autores, publicaciones y afiliaciones
            db.arrayOfStrings2DB(autores,"authors"); 
            db.arrayOfStrings2DB(publicaciones,"publication");
            db.affiliations2DB(filiaciones); 

            //FASE 3.2 Generación de autor-filiación, usando el conjunto "autFil" y los indices de autor y filiación   
            for (String af : autFil) {
                String[] autAndFil=separaAutorYFiliacion(af);
                db.addAuthorFiliation(1+Arrays.binarySearch(autores, autAndFil[0].trim()),1+Arrays.binarySearch(filiaciones, autAndFil[1].trim()));
                }
            db.flushAuthorFiliation();

            muestraTiempo(t0,"tras la escritura de índices en la base de datos");

            //FASE 3.3 Generación de tablas de papel-autor, info y de referencias    
            for (Entry<Integer, Referencia> e : Referencia.getAlmacen().entrySet()){
                for (String a:e.getValue().getAutorFiliacionList())
                    db.addPaperAuth(e.getKey(), 1+Arrays.binarySearch(autores, separaAutorYFiliacion(a)[0].trim()));
                db.addInfo(e.getKey(), e.getValue(),1+Arrays.binarySearch(publicaciones, e.getValue().getPublicacion()));
                if (e.getValue().getReferencias()!=null) db.addReferences(e.getKey(), e.getValue().getReferencias().split(" +"));
            }    
            db.flushPaperAuth();            
            db.flushReferences();            
            db.flushInfo();            

            muestraTiempo(t0,"tras la escritura de papel-autor, info, referencias");
        }
    }

    private static String[] separaAutorYFiliacion(String s) {
        return s.replaceFirst("(.*?)\"(.*?)\".*", "$1<sep>$2").split("<sep>");
    }
    
    private static void muestraTiempo(Long t0, String msg) {
        System.out.printf("%.3f secs. %s\n",(System.currentTimeMillis()-t0)/1000.0, msg);
    }
    

}